/*
 * File access convenience functions and types.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/types.h>

#include <systypes/base.h>



EXTERN_C_BEGIN

/* File access abstraction. */

typedef struct
{
  /* File object reference. */

  l4_cap_idx_t ref;

  /* Mapped memory accessing a file region. */

  char *memory;

  /* File region parameters. */

  offset_t start_pos, end_pos;  /* start and end positions of region */
  offset_t data_end;            /* amount of data in the region */

  /* Total size of file. */

  offset_t size;

} file_t;



/* File operations. */

void file_close(file_t *file);
long file_open(file_t *file, const char *filename, l4_cap_idx_t server);

/* File lifecycle operations. */

long file_context(file_t *file, l4_cap_idx_t server);
long file_context_open(file_t *file, file_t *context);
void file_init(file_t *file);

/* File access region operations. */

long file_mmap(file_t *file, offset_t position, offset_t length);
offset_t file_populated_span(file_t *file);
offset_t file_span(file_t *file);

/* Convenience functions. */

char *file_string_get(file_t *file, offset_t offset);
int file_string_set(file_t *file, const char *data, offset_t offset, offset_t *written);

EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
